;;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;;;
;;;     			     ANSIL
;;; 		      Advanced Network Simulator In Lisp
;;; 		      ----------------------------------
;;;
;;;				  Written By
;;;			       Peter J. Angeline
;;;			      Gregory M. Saunders
;;;
;;;			   The Ohio State University
;;;		       Laboratory for AI Research (LAIR)
;;;			     Columbus, Ohio, 43210
;;;
;;;			   Copyright (c) 1991, 1992
;;;
;;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;;; ---------------------------------------------------------------------- 
;;; File: hopfield.lisp -- an example.
;;;
;;; Author: Gregory M Saunders
;;;
;;; License: This code may be distributed free of charge.  This code may not be
;;; incorporated into any production software of any type or any code which is
;;; sold without the express written consent of the authors.  Modifications and
;;; inclusions for other types of software are permitted, provided that this
;;; header remains unchanged and included in all subsequent versions of the
;;; code.
;;;
;;; Disclaimer: The authors of this code make no claims as to its
;;; validity, and are in no way responsible for any errors contained within.
;;; 
;;; File description:
;;;
;;; Creation date: March 19, 1992, Version 2.01
;;; ----------------------------------------------------------------------
;;;
;;;
;;;                         Change Log
;;;                         ----------
;;;
;;; ----------------------------------------------------------------------

(defun setup (&key (n 10) (m 2))
  (format t "Setting up hopfield net with ~a nodes and ~a patterns...~%" n m)
  (setf net (create-network n :type 'hopfield))
  (setf I (identity-matrix (1+ (network-maxnode net))))
  (setf (network-tset net)
	(loop for i from 1 to m collect
	      (coerce-vector 
	       (loop for j from 1 to n collect
		     (if (zerop (random 2)) -1 1)))))
  (hopfield-train net))

(defun store (net x &aux (w (network-weights net)))
  (setf x (coerce-vector x))
  (unless (loop for y across x always
		(or (= y -1d0)
		    (= y  1d0)))
	  (error "can only store vector over +1/-1"))
  (+mm w (outer x x) :res w)
  (-mm w I :res w))

(defun mysign (x)
  (if (< x 0d0) -1d0 1d0))

(defun retrieve (net vec &key (num 50)
		     &aux 
		     (x (copy.v vec))
		     (y (create-array (length x) :initial-element -999)))
  (loop until (or (zerop num)
		  (=vv y x))
	do
	(move.vv x y)
	(setf x (fpass net x :num 1))
	(setf x (coerce-vector (loop for xx across x collect (mysign xx))))
;	(format t "~a~%" x)
	)
  x)

(defun hopfield-train (net)
  (zero-matrix (network-weights net))
  (loop for x in (network-tset net) do
	(format t "  Storing ~a~%" (format-vector x))
	(store net x)))

(defun test-all (net &aux y)
  (loop for x in (network-tset net)
	for i = 0 then (1+ i) do
	(format t "Retrieving #~a~%" i)
	(setf y (retrieve net x))
	(format t "  x = ~a~%" (format-vector x))
	(format t "  y = ~a~%" (format-vector y))
	(format t " ===> ~a~%~%" (=vv x y))))

(defun demo (&key (n 100) (m 10))
  (setup :n n :m m)
  (format t "~%")
  (test-all net))

(demo)
