;;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;;;
;;;     			     ANSIL
;;; 		      Advanced Network Simulator In Lisp
;;; 		      ----------------------------------
;;;
;;;				  Written By
;;;			       Peter J. Angeline
;;;			      Gregory M. Saunders
;;;
;;;			   The Ohio State University
;;;		       Laboratory for AI Research (LAIR)
;;;			     Columbus, Ohio, 43210
;;;
;;;			   Copyright (c) 1991, 1992
;;;
;;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;;; ---------------------------------------------------------------------- 
;;; File: cl.lisp - compile-load the ANSIL system
;;;
;;; Authors: Peter J Angeline, Gregory M Saunders
;;;
;;; License: This code may be distributed free of charge.  This code may not be
;;; incorporated into any production software of any type or any code which is
;;; sold without the express written consent of the authors.  Modifications and
;;; inclusions for other types of software are permitted, provided that this
;;; header remains unchanged and included in all subsequent versions of the
;;; code.
;;;
;;; Disclaimer: The authors of this code make no claims as to its
;;; validity, and are in no way responsible for any errors contained within.
;;; 
;;; File description: This file contains implementations of linear algebra
;;; routines which are implemented in C routines and called by Franz Allegro
;;; Common Lisp.  This file is specifc to Franz Allegro Common Lisp and will
;;; not work with other common lisps.  The file linear1.lisp should be used as
;;; a replacement for this file when using a different common lisp. the file
;;; with the C routines, linear2.o, should be placed in the same directory.
;;;
;;; Creation date: March 17, 1992, Version 2.01
;;; ----------------------------------------------------------------------
;;;
;;;
;;;                         Change Log
;;;                         ----------
;;;
;;; ----------------------------------------------------------------------


;;; ---------------------------------------------------------------------------
;;;
;;; Optimization
;;;
;;; ---------------------------------------------------------------------------

(declaim (optimize (speed 0) (safety 3) (debug 3) (space 3)))
;(declaim (optimize (speed 3) (safety 0) (debug 0) (space 0)))

;;; ---------------------------------------------------------------------------
;;;
;;; Variables
;;;
;;; ---------------------------------------------------------------------------

(defvar *ansil-path* "~ansil/ver2.02/source/")
(defvar *always-compile* nil)
(defvar *always-compile-files* nil) ;'("net1" "net2" "raam")
(defvar *c* t)
(defvar *c-file* "linear2")
(defvar *fasl-extension* ".fasl")

(defvar *default-squash-function* 1d0) ;sigmoid


;;; ---------------------------------------------------------------------------
;;;
;;; Loop
;;;
;;; ---------------------------------------------------------------------------

(format t "Compile-loading the ANSIL System...~%")
(require :loop)


;;; ---------------------------------------------------------------------------
;;;
;;; C files
;;;
;;; ---------------------------------------------------------------------------

(defun cc ()
  (when (or (not (file-author (concatenate 'string *ansil-path* *c-file* ".o"))) ;existence
	    (file-older-p 
	     (concatenate 'string *ansil-path* *c-file* ".o")
	     (concatenate 'string *ansil-path* *c-file* ".c")))
	(format t "Compiling ~a.c..." *c-file*)
	(run-shell-command 
	 (concatenate 'string
		      "cc "
		      *ansil-path*
		      *c-file*
		      ".c -c -lm -o "
		      *ansil-path*
		      *c-file*
		      ".o"))
	(format t "done~%")))

(when *c*
      (use-package :ff)
      (require :foreign)
      (cc))


;;; ---------------------------------------------------------------------------
;;;
;;; Compile-load in ansil
;;;
;;; ---------------------------------------------------------------------------

(loop for x in 
      `("generic"
	"random" 
	,(if *c* "linear2" "linear1")
	"math"
	"net1"
	"net2"
	"raam"
	"show")
      do
      (if (or *always-compile*
	      (member x *always-compile-files* :test #'equal))
	  (compile-file (concatenate 'string *ansil-path* x))
	(compile-file-if-needed (concatenate 'string *ansil-path* x)))
      (load (concatenate 'string *ansil-path* x)))

(format t "Compile-load Complete~%")
