(defpackage mbp)

(defun setup1 ()
  (setf net (create-network 5 :type 'feed-forward :layers '(2 2 1)))
  (set-weight net 0 3 -.1d0)
  (set-weight net 1 3 .4d0)
  (set-weight net 2 3 .2d0)
  (set-weight net 0 4 .2d0)
  (set-weight net 1 4 -.6d0)
  (set-weight net 2 4 -.12d0)
  (set-weight net 0 5 .5d0)
  (set-weight net 3 5 -.3d0)
  (set-weight net 4 5 -.4d0)
  (add-pattern net '(1 0 0) '(0))
  (add-pattern net '(1 0 1) '(1))
  (add-pattern net '(1 1 0) '(1))
  (add-pattern net '(1 1 1) '(0))
;  (show-network net)
  )

(defun setup2 ()
  (setf net (create-network '(2 2 1)))
  (setf (aref (car (network-weights net)) 0 0) -.1d0)
  (setf (aref (car (network-weights net)) 1 0) .4d0)
  (setf (aref (car (network-weights net)) 2 0) .2d0)
  (setf (aref (car (network-weights net)) 0 1) .2d0)
  (setf (aref (car (network-weights net)) 1 1) -.6d0)
  (setf (aref (car (network-weights net)) 2 1) -.12d0)
  (setf (aref (cadr (network-weights net)) 0 0) .5d0)
  (setf (aref (cadr (network-weights net)) 1 0) -.3d0)
  (setf (aref (cadr (network-weights net)) 2 0) -.4d0)
  (add-pattern net '(0 0) '(0))
  (add-pattern net '(0 1) '(1))
  (add-pattern net '(1 0) '(1))
  (add-pattern net '(1 1) '(0)))

(defun set-targ (x y)
  (setf in (coerce-vector x))
  (setf targ (coerce-vector y)))

(set-targ '(1 1 1) '(1))


;;; ---------------------------------------------------------------------------
;;;
;;; new stuff
;;;
;;; ---------------------------------------------------------------------------

(defun test1 (&key (setup nil))
  (format t "Testing~%")
  (if setup (setup1))
  (fpass net in)
  (format t "Fpassing ~a~%  ===> Act = ~a~%~%"
	  in (network-activations net))
  (bpass net :target targ)
  (format t "Target = ~a~%  ===> Deltas =  ~a~%"
	  targ (network-deltas net))
  (compute-dweights net)
  (format t "dweights~%")
  (show-matrix (matrix-apply* nil
			      (%*ms nil (network-dweights net) (network-rate net))
			      (network-free-weights net)))
  (update-weights net)
  (format t "New weights~%")
  (show-matrix (network-weights net))
  (update-pdweights net)
  (format t "Prev dweights~%")
  (show-matrix (network-pdweights net))
  )

(defun train1 (net in targ)
  (fpass net in)
  (bpass net :target targ)
  (compute-dweights net))

(defun update1 ()
  (update-weights net)
  (update-pdweights net))

(defun show1 ()
  (fpass net in)
  (format t "Fpassing ~a~%  ===> Act = ~a~%~%"
	  in (network-activations net))
  (bpass net :target targ)
  (format t "Target = ~a~%  ===> Deltas =  ~a~%"
	  targ (network-deltas net))
  (format t "dweights~%")
  (show-matrix (matrix-apply* nil
			      (%*ms nil (network-dweights net) (network-rate net))
			      (network-free-weights net)))
;  (update1)
  (format t "New weights~%")
  (show-matrix (network-weights net) :digits 8)
  (format t "Prev dweights~%")
  (show-matrix (network-pdweights net)))

(defun foo1 ()
  (loop for x in (network-tset net) do
	(format t ".")
	(train1 net (car x) (cadr x)))
  (update1)
  (show1))

(defun bar1 ()
  (loop for x in (network-tset net) do
	(format t ".")
	(train1 net (car x) (cadr x)))
  (update1))


;;; ---------------------------------------------------------------------------
;;;
;;; Old stuff
;;;
;;; ---------------------------------------------------------------------------

(defun test2 (&key (setup nil))
  (format t "Testing~%")
  (if setup (setup2))
  (fpass net (cdr-vector-array in))
  (format t "Fpassing ~a~%  ===> Act = ~%" in)
  (loop for x in (cdr (network-activations net)) do
	(format t "   ~a~%" x))
  (compute-output-errors net targ 0d0)
  (compute-hidden-errors net)
  (format t "~%Target = ~a  ===> Deltas = ~%" targ)
  (loop for x in (network-errors net) do
	(format t "  ~a~%" x))
  (compute-deltas net)
  (format t "~%dweights~%")
  (loop for x in (network-deltas net) do
	(print-matrix x))
  (update-weights net)
  (format t "New weights~%")
  (loop for x in (network-weights net) do
	(print-matrix x))
  (update-prev-deltas net)
  (loop for x in (network-deltas net) do
	(mbp::zero-matrix x))
  (format t "Prev dweights~%")
  (loop for x in (network-prev-deltas net) do
	(print-matrix x)))

(defun train2 (net in targ)
  (fpass net in)
  (compute-output-errors net targ 0d0)
  (compute-hidden-errors net)
  (compute-deltas net))

(defun update2 ()
  (update-weights net)
  (update-prev-deltas net)
  (loop for x in (network-deltas net) do
	(mbp::zero-matrix x)))

(defun show2 ()
  (fpass net (cdr-vector-array in))
  (format t "Fpassing ~a~%  ===> Act = ~%" in)
  (loop for x in (cdr (network-activations net)) do
	(format t "   ~a~%" x))
  (compute-output-errors net targ 0d0)
  (compute-hidden-errors net)
  (format t "~%Target = ~a  ===> Deltas = ~%" targ)
  (loop for x in (network-errors net) do
	(format t "  ~a~%" x))
  (format t "~%dweights~%")
  (loop for x in (network-deltas net) do
	(print-matrix x))
;  (update2)
  (format t "New weights~%")
  (loop for x in (network-weights net) do
	(print-matrix x :digits 8))
  (format t "Prev dweights~%")
  (loop for x in (network-prev-deltas net) do
	(print-matrix x)))

(defun foo2 ()
  (loop for x in (network-patterns net) do
	(format t ".")
	(train2 net (car x) (cadr x)))
  (update2)
  (show2))

(defun bar2 ()
  (loop for x in (network-patterns net) do
	(format t ".")
	(train2 net (car x) (cadr x)))
  (update2))




(setf x (coerce-vector '(1 2 3)))
(setf z (coerce-vector '(10)))
(setf mm (create-array '(3 3)))
(setf (aref mm 0 0) 1d0)
(setf (aref mm 0 1) 2d0)
(setf (aref mm 0 2) 3d0)
(setf (aref mm 1 0) 4d0)
(setf (aref mm 1 1) 5d0)
(setf (aref mm 1 2) 6d0)
(setf (aref mm 2 0) 7d0)
(setf (aref mm 2 1) 8d0)
(setf (aref mm 2 2) 9d0)

(setf xx (coerce-vector '(1 2 3 4 5)))
(setf yy (coerce-vector '(-1 -2 -3 -4 -5)))
(setf f (coerce-vector '(1 1 1 1 1)))
